@@ -1,14 +1,8 @@
-"""
-    sphinx.builders.dummy
-    ~~~~~~~~~~~~~~~~~~~~~
+"""Do syntax checks, but no writing."""
 
-    Do syntax checks, but no writing.
+from __future__ import annotations
 
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
-
-from typing import Any, Dict, Set
+from typing import Any
 
 from docutils.nodes import Node
 
@@ -26,13 +20,13 @@ class DummyBuilder(Builder):
     def init(self) -> None:
         pass
 
-    def get_outdated_docs(self) -> Set[str]:
+    def get_outdated_docs(self) -> set[str]:
         return self.env.found_docs
 
-    def get_target_uri(self, docname: str, typ: str = None) -> str:
+    def get_target_uri(self, docname: str, typ: str | None = None) -> str:
         return ''
 
-    def prepare_writing(self, docnames: Set[str]) -> None:
+    def prepare_writing(self, docnames: set[str]) -> None:
         pass
 
     def write_doc(self, docname: str, doctree: Node) -> None:
@@ -42,7 +36,7 @@ class DummyBuilder(Builder):
         pass
 
 
-def setup(app: Sphinx) -> Dict[str, Any]:
+def setup(app: Sphinx) -> dict[str, Any]:
     app.add_builder(DummyBuilder)
 
     return {
