@@ -12,6 +12,8 @@ Example Makefile rule::
            sphinx-autogen -o source/generated source/*.rst
 """
 
+from __future__ import annotations
+
 import argparse
 import inspect
 import locale
@@ -20,9 +22,8 @@ import pkgutil
 import pydoc
 import re
 import sys
-from gettext import NullTranslations
 from os import path
-from typing import Any, Dict, List, NamedTuple, Sequence, Set, Tuple, Type
+from typing import TYPE_CHECKING, Any, NamedTuple, Sequence
 
 from jinja2 import TemplateNotFound
 from jinja2.sandbox import SandboxedEnvironment
@@ -34,8 +35,12 @@ from sphinx.builders import Builder
 from sphinx.config import Config
 from sphinx.ext.autodoc import Documenter
 from sphinx.ext.autodoc.importer import import_module
-from sphinx.ext.autosummary import (ImportExceptionGroup, get_documenter, import_by_name,
-                                    import_ivar_by_name)
+from sphinx.ext.autosummary import (
+    ImportExceptionGroup,
+    get_documenter,
+    import_by_name,
+    import_ivar_by_name,
+)
 from sphinx.locale import __
 from sphinx.pycode import ModuleAnalyzer, PycodeError
 from sphinx.registry import SphinxComponentRegistry
@@ -44,6 +49,9 @@ from sphinx.util.inspect import getall, safe_getattr
 from sphinx.util.osutil import ensuredir
 from sphinx.util.template import SphinxTemplateLoader
 
+if TYPE_CHECKING:
+    from gettext import NullTranslations
+
 logger = logging.getLogger(__name__)
 
 
@@ -53,7 +61,7 @@ class DummyApplication:
     def __init__(self, translator: NullTranslations) -> None:
         self.config = Config()
         self.registry = SphinxComponentRegistry()
-        self.messagelog: List[str] = []
+        self.messagelog: list[str] = []
         self.srcdir = "/"
         self.translator = translator
         self.verbosity = 0
@@ -77,15 +85,21 @@ class AutosummaryEntry(NamedTuple):
 
 
 def setup_documenters(app: Any) -> None:
-    from sphinx.ext.autodoc import (AttributeDocumenter, ClassDocumenter, DataDocumenter,
-                                    DecoratorDocumenter, ExceptionDocumenter,
-                                    FunctionDocumenter, MethodDocumenter, ModuleDocumenter,
-                                    NewTypeAttributeDocumenter, NewTypeDataDocumenter,
-                                    PropertyDocumenter)
-    documenters: List[Type[Documenter]] = [
+    from sphinx.ext.autodoc import (
+        AttributeDocumenter,
+        ClassDocumenter,
+        DataDocumenter,
+        DecoratorDocumenter,
+        ExceptionDocumenter,
+        FunctionDocumenter,
+        MethodDocumenter,
+        ModuleDocumenter,
+        PropertyDocumenter,
+    )
+    documenters: list[type[Documenter]] = [
         ModuleDocumenter, ClassDocumenter, ExceptionDocumenter, DataDocumenter,
-        FunctionDocumenter, MethodDocumenter, NewTypeAttributeDocumenter,
-        NewTypeDataDocumenter, AttributeDocumenter, DecoratorDocumenter, PropertyDocumenter,
+        FunctionDocumenter, MethodDocumenter,
+        AttributeDocumenter, DecoratorDocumenter, PropertyDocumenter,
     ]
     for documenter in documenters:
         app.registry.add_documenter(documenter.objtype, documenter)
@@ -117,7 +131,7 @@ class AutosummaryRenderer:
             self.env.add_extension("jinja2.ext.i18n")
             self.env.install_gettext_translations(app.translator)
 
-    def render(self, template_name: str, context: Dict) -> str:
+    def render(self, template_name: str, context: dict) -> str:
         """Render a template file."""
         try:
             template = self.env.get_template(template_name)
@@ -153,7 +167,7 @@ class ModuleScanner:
                            name, exc, type='autosummary')
             return False
 
-    def scan(self, imported_members: bool) -> List[str]:
+    def scan(self, imported_members: bool) -> list[str]:
         members = []
         try:
             analyzer = ModuleAnalyzer.for_module(self.object.__name__)
@@ -174,7 +188,7 @@ class ModuleScanner:
             try:
                 if ('', name) in attr_docs:
                     imported = False
-                elif inspect.ismodule(value):
+                elif inspect.ismodule(value):  # NoQA: SIM114
                     imported = True
                 elif safe_getattr(value, '__module__') != self.object.__name__:
                     imported = True
@@ -184,14 +198,14 @@ class ModuleScanner:
                 imported = False
 
             respect_module_all = not self.app.config.autosummary_ignore_module_all
-            if imported_members:
+            if (
                 # list all members up
-                members.append(name)
-            elif imported is False:
+                imported_members
                 # list not-imported members
-                members.append(name)
-            elif '__all__' in dir(self.object) and respect_module_all:
+                or imported is False
                 # list members that have __all__ set
+                or (respect_module_all and '__all__' in dir(self.object))
+            ):
                 members.append(name)
 
         return members
@@ -211,8 +225,9 @@ def members_of(obj: Any, conf: Config) -> Sequence[str]:
 def generate_autosummary_content(name: str, obj: Any, parent: Any,
                                  template: AutosummaryRenderer, template_name: str,
                                  imported_members: bool, app: Any,
-                                 recursive: bool, context: Dict,
-                                 modname: str = None, qualname: str = None) -> str:
+                                 recursive: bool, context: dict,
+                                 modname: str | None = None,
+                                 qualname: str | None = None) -> str:
     doc = get_documenter(app, obj, parent)
 
     def skip_member(obj: Any, name: str, objtype: str) -> bool:
@@ -225,11 +240,11 @@ def generate_autosummary_content(name: str, obj: Any, parent: Any,
                            name, exc, type='autosummary')
             return False
 
-    def get_class_members(obj: Any) -> Dict[str, Any]:
+    def get_class_members(obj: Any) -> dict[str, Any]:
         members = sphinx.ext.autodoc.get_class_members(obj, [qualname], safe_getattr)
         return {name: member.object for name, member in members.items()}
 
-    def get_module_members(obj: Any) -> Dict[str, Any]:
+    def get_module_members(obj: Any) -> dict[str, Any]:
         members = {}
         for name in members_of(obj, app.config):
             try:
@@ -238,17 +253,17 @@ def generate_autosummary_content(name: str, obj: Any, parent: Any,
                 continue
         return members
 
-    def get_all_members(obj: Any) -> Dict[str, Any]:
+    def get_all_members(obj: Any) -> dict[str, Any]:
         if doc.objtype == "module":
             return get_module_members(obj)
         elif doc.objtype == "class":
             return get_class_members(obj)
         return {}
 
-    def get_members(obj: Any, types: Set[str], include_public: List[str] = [],
-                    imported: bool = True) -> Tuple[List[str], List[str]]:
-        items: List[str] = []
-        public: List[str] = []
+    def get_members(obj: Any, types: set[str], include_public: list[str] = [],
+                    imported: bool = True) -> tuple[list[str], list[str]]:
+        items: list[str] = []
+        public: list[str] = []
 
         all_members = get_all_members(obj)
         for name, value in all_members.items():
@@ -270,7 +285,7 @@ def generate_autosummary_content(name: str, obj: Any, parent: Any,
                             public.append(name)
         return public, items
 
-    def get_module_attrs(members: Any) -> Tuple[List[str], List[str]]:
+    def get_module_attrs(members: Any) -> tuple[list[str], list[str]]:
         """Find module attributes with docstrings."""
         attrs, public = [], []
         try:
@@ -285,9 +300,17 @@ def generate_autosummary_content(name: str, obj: Any, parent: Any,
             pass    # give up if ModuleAnalyzer fails to parse code
         return public, attrs
 
-    def get_modules(obj: Any) -> Tuple[List[str], List[str]]:
-        items: List[str] = []
+    def get_modules(
+            obj: Any,
+            skip: Sequence[str],
+            public_members: Sequence[str] | None = None) -> tuple[list[str], list[str]]:
+        items: list[str] = []
+        public: list[str] = []
         for _, modname, _ispkg in pkgutil.iter_modules(obj.__path__):
+
+            if modname in skip:
+                # module was overwritten in __init__.py, so not accessible
+                continue
             fullname = name + '.' + modname
             try:
                 module = import_module(fullname)
@@ -297,15 +320,24 @@ def generate_autosummary_content(name: str, obj: Any, parent: Any,
                 pass
 
             items.append(fullname)
-        public = [x for x in items if not x.split('.')[-1].startswith('_')]
+            if public_members is not None:
+                if modname in public_members:
+                    public.append(fullname)
+            else:
+                if not modname.startswith('_'):
+                    public.append(fullname)
         return public, items
 
-    ns: Dict[str, Any] = {}
+    ns: dict[str, Any] = {}
     ns.update(context)
 
     if doc.objtype == 'module':
         scanner = ModuleScanner(app, obj)
         ns['members'] = scanner.scan(imported_members)
+
+        respect_module_all = not app.config.autosummary_ignore_module_all
+        imported_members = imported_members or ('__all__' in dir(obj) and respect_module_all)
+
         ns['functions'], ns['all_functions'] = \
             get_members(obj, {'function'}, imported=imported_members)
         ns['classes'], ns['all_classes'] = \
@@ -316,7 +348,36 @@ def generate_autosummary_content(name: str, obj: Any, parent: Any,
             get_module_attrs(ns['members'])
         ispackage = hasattr(obj, '__path__')
         if ispackage and recursive:
-            ns['modules'], ns['all_modules'] = get_modules(obj)
+            # Use members that are not modules as skip list, because it would then mean
+            # that module was overwritten in the package namespace
+            skip = (
+                ns["all_functions"]
+                + ns["all_classes"]
+                + ns["all_exceptions"]
+                + ns["all_attributes"]
+            )
+
+            # If respect_module_all and module has a __all__ attribute, first get
+            # modules that were explicitly imported. Next, find the rest with the
+            # get_modules method, but only put in "public" modules that are in the
+            # __all__ list
+            #
+            # Otherwise, use get_modules method normally
+            if respect_module_all and '__all__' in dir(obj):
+                imported_modules, all_imported_modules = \
+                    get_members(obj, {'module'}, imported=True)
+                skip += all_imported_modules
+                imported_modules = [name + '.' + modname for modname in imported_modules]
+                all_imported_modules = \
+                    [name + '.' + modname for modname in all_imported_modules]
+                public_members = getall(obj)
+            else:
+                imported_modules, all_imported_modules = [], []
+                public_members = None
+
+            modules, all_modules = get_modules(obj, skip=skip, public_members=public_members)
+            ns['modules'] = imported_modules + modules
+            ns["all_modules"] = all_imported_modules + all_modules
     elif doc.objtype == 'class':
         ns['members'] = dir(obj)
         ns['inherited_members'] = \
@@ -351,8 +412,8 @@ def generate_autosummary_content(name: str, obj: Any, parent: Any,
         return template.render(doc.objtype, ns)
 
 
-def generate_autosummary_docs(sources: List[str], output_dir: str = None,
-                              suffix: str = '.rst', base_path: str = None,
+def generate_autosummary_docs(sources: list[str], output_dir: str | None = None,
+                              suffix: str = '.rst', base_path: str | None = None,
                               imported_members: bool = False, app: Any = None,
                               overwrite: bool = True, encoding: str = 'utf-8') -> None:
     showed_sources = sorted(sources)
@@ -400,16 +461,16 @@ def generate_autosummary_docs(sources: List[str], output_dir: str = None,
                 qualname = name.replace(modname + ".", "")
             except ImportError as exc2:
                 if exc2.__cause__:
-                    exceptions: List[BaseException] = exc.exceptions + [exc2.__cause__]
+                    exceptions: list[BaseException] = exc.exceptions + [exc2.__cause__]
                 else:
                     exceptions = exc.exceptions + [exc2]
 
-                errors = list({"* %s: %s" % (type(e).__name__, e) for e in exceptions})
+                errors = list({f"* {type(e).__name__}: {e}" for e in exceptions})
                 logger.warning(__('[autosummary] failed to import %s.\nPossible hints:\n%s'),
                                entry.name, '\n'.join(errors))
                 continue
 
-        context: Dict[str, Any] = {}
+        context: dict[str, Any] = {}
         if app:
             context.update(app.config.autosummary_context)
 
@@ -424,7 +485,7 @@ def generate_autosummary_docs(sources: List[str], output_dir: str = None,
 
             if content == old_content:
                 continue
-            elif overwrite:  # content has changed
+            if overwrite:  # content has changed
                 with open(filename, 'w', encoding=encoding) as f:
                     f.write(content)
                 new_files.append(filename)
@@ -443,12 +504,12 @@ def generate_autosummary_docs(sources: List[str], output_dir: str = None,
 
 # -- Finding documented entries in files ---------------------------------------
 
-def find_autosummary_in_files(filenames: List[str]) -> List[AutosummaryEntry]:
+def find_autosummary_in_files(filenames: list[str]) -> list[AutosummaryEntry]:
     """Find out what items are documented in source/*.rst.
 
     See `find_autosummary_in_lines`.
     """
-    documented: List[AutosummaryEntry] = []
+    documented: list[AutosummaryEntry] = []
     for filename in filenames:
         with open(filename, encoding='utf-8', errors='ignore') as f:
             lines = f.read().splitlines()
@@ -456,7 +517,9 @@ def find_autosummary_in_files(filenames: List[str]) -> List[AutosummaryEntry]:
     return documented
 
 
-def find_autosummary_in_docstring(name: str, filename: str = None) -> List[AutosummaryEntry]:
+def find_autosummary_in_docstring(
+    name: str, filename: str | None = None,
+) -> list[AutosummaryEntry]:
     """Find out what items are documented in the given object's docstring.
 
     See `find_autosummary_in_lines`.
@@ -468,16 +531,17 @@ def find_autosummary_in_docstring(name: str, filename: str = None) -> List[Autos
     except AttributeError:
         pass
     except ImportExceptionGroup as exc:
-        errors = list({"* %s: %s" % (type(e).__name__, e) for e in exc.exceptions})
-        print('Failed to import %s.\nPossible hints:\n%s' % (name, '\n'.join(errors)))
+        errors = '\n'.join({f"* {type(e).__name__}: {e}" for e in exc.exceptions})
+        print(f'Failed to import {name}.\nPossible hints:\n{errors}')
     except SystemExit:
         print("Failed to import '%s'; the module executes module level "
               "statement and it might call sys.exit()." % name)
     return []
 
 
-def find_autosummary_in_lines(lines: List[str], module: str = None, filename: str = None
-                              ) -> List[AutosummaryEntry]:
+def find_autosummary_in_lines(
+    lines: list[str], module: str | None = None, filename: str | None = None,
+) -> list[AutosummaryEntry]:
     """Find out what items appear in autosummary:: directives in the
     given lines.
 
@@ -498,10 +562,10 @@ def find_autosummary_in_lines(lines: List[str], module: str = None, filename: st
     toctree_arg_re = re.compile(r'^\s+:toctree:\s*(.*?)\s*$')
     template_arg_re = re.compile(r'^\s+:template:\s*(.*?)\s*$')
 
-    documented: List[AutosummaryEntry] = []
+    documented: list[AutosummaryEntry] = []
 
     recursive = False
-    toctree: str = None
+    toctree: str | None = None
     template = None
     current_module = module
     in_autosummary = False
@@ -537,7 +601,7 @@ def find_autosummary_in_lines(lines: List[str], module: str = None, filename: st
                     name = name[1:]
                 if current_module and \
                    not name.startswith(current_module + '.'):
-                    name = "%s.%s" % (current_module, name)
+                    name = f"{current_module}.{name}"
                 documented.append(AutosummaryEntry(name, toctree, template, recursive))
                 continue
 
@@ -617,12 +681,11 @@ The format of the autosummary directive is documented in the
     return parser
 
 
-def main(argv: List[str] = sys.argv[1:]) -> None:
-    sphinx.locale.setlocale(locale.LC_ALL, '')
-    sphinx.locale.init_console(os.path.join(package_dir, 'locale'), 'sphinx')
-    translator, _ = sphinx.locale.init([], None)
+def main(argv: list[str] = sys.argv[1:]) -> None:
+    locale.setlocale(locale.LC_ALL, '')
+    sphinx.locale.init_console()
 
-    app = DummyApplication(translator)
+    app = DummyApplication(sphinx.locale.get_translator())
     logging.setup(app, sys.stdout, sys.stderr)  # type: ignore
     setup_documenters(app)
     args = get_parser().parse_args(argv)
