@@ -1,35 +1,32 @@
-"""
-    sphinx.pycode
-    ~~~~~~~~~~~~~
+"""Utilities parsing and analyzing Python code."""
 
-    Utilities parsing and analyzing Python code.
+from __future__ import annotations
 
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
-
-import re
 import tokenize
-import warnings
-from collections import OrderedDict
 from importlib import import_module
-from inspect import Signature
-from io import StringIO
 from os import path
-from typing import IO, Any, Dict, List, Optional, Tuple
-from zipfile import ZipFile
+from typing import TYPE_CHECKING, Any
 
-from sphinx.deprecation import RemovedInSphinx50Warning
 from sphinx.errors import PycodeError
 from sphinx.pycode.parser import Parser
 
+if TYPE_CHECKING:
+    from inspect import Signature
+
 
 class ModuleAnalyzer:
+    annotations: dict[tuple[str, str], str]
+    attr_docs: dict[tuple[str, str], list[str]]
+    finals: list[str]
+    overloads: dict[str, list[Signature]]
+    tagorder: dict[str, int]
+    tags: dict[str, tuple[str, int, int]]
+
     # cache for analyzer objects -- caches both by module and file name
-    cache: Dict[Tuple[str, str], Any] = {}
+    cache: dict[tuple[str, str], Any] = {}
 
     @staticmethod
-    def get_module_source(modname: str) -> Tuple[Optional[str], Optional[str]]:
+    def get_module_source(modname: str) -> tuple[str | None, str | None]:
         """Try to find the source code for a module.
 
         Returns ('filename', 'source'). One of it can be None if
@@ -66,49 +63,31 @@ class ModuleAnalyzer:
                 filename += 'w'
         elif not filename.lower().endswith(('.py', '.pyw')):
             raise PycodeError('source is not a .py file: %r' % filename)
-        elif ('.egg' + path.sep) in filename:
-            pat = '(?<=\\.egg)' + re.escape(path.sep)
-            eggpath, _ = re.split(pat, filename, 1)
-            if path.isfile(eggpath):
-                return filename, None
 
         if not path.isfile(filename):
             raise PycodeError('source file is not present: %r' % filename)
         return filename, None
 
     @classmethod
-    def for_string(cls, string: str, modname: str, srcname: str = '<string>'
-                   ) -> "ModuleAnalyzer":
-        return cls(StringIO(string), modname, srcname)
+    def for_string(cls, string: str, modname: str, srcname: str = '<string>',
+                   ) -> ModuleAnalyzer:
+        return cls(string, modname, srcname)
 
     @classmethod
-    def for_file(cls, filename: str, modname: str) -> "ModuleAnalyzer":
+    def for_file(cls, filename: str, modname: str) -> ModuleAnalyzer:
         if ('file', filename) in cls.cache:
             return cls.cache['file', filename]
         try:
             with tokenize.open(filename) as f:
-                obj = cls(f, modname, filename)
-                cls.cache['file', filename] = obj
+                string = f.read()
+            obj = cls(string, modname, filename)
+            cls.cache['file', filename] = obj
         except Exception as err:
-            if '.egg' + path.sep in filename:
-                obj = cls.cache['file', filename] = cls.for_egg(filename, modname)
-            else:
-                raise PycodeError('error opening %r' % filename, err) from err
+            raise PycodeError('error opening %r' % filename, err) from err
         return obj
 
     @classmethod
-    def for_egg(cls, filename: str, modname: str) -> "ModuleAnalyzer":
-        SEP = re.escape(path.sep)
-        eggpath, relpath = re.split('(?<=\\.egg)' + SEP, filename)
-        try:
-            with ZipFile(eggpath) as egg:
-                code = egg.read(relpath).decode()
-                return cls.for_string(code, modname, filename)
-        except Exception as exc:
-            raise PycodeError('error opening %r' % filename, exc) from exc
-
-    @classmethod
-    def for_module(cls, modname: str) -> "ModuleAnalyzer":
+    def for_module(cls, modname: str) -> ModuleAnalyzer:
         if ('module', modname) in cls.cache:
             entry = cls.cache['module', modname]
             if isinstance(entry, PycodeError):
@@ -127,38 +106,25 @@ class ModuleAnalyzer:
         cls.cache['module', modname] = obj
         return obj
 
-    def __init__(self, source: IO, modname: str, srcname: str) -> None:
+    def __init__(self, source: str, modname: str, srcname: str) -> None:
         self.modname = modname  # name of the module
         self.srcname = srcname  # name of the source file
 
         # cache the source code as well
-        self.code = source.read()
+        self.code = source
 
-        # will be filled by analyze()
-        self.annotations: Dict[Tuple[str, str], str] = None
-        self.attr_docs: Dict[Tuple[str, str], List[str]] = None
-        self.finals: List[str] = None
-        self.overloads: Dict[str, List[Signature]] = None
-        self.tagorder: Dict[str, int] = None
-        self.tags: Dict[str, Tuple[str, int, int]] = None
         self._analyzed = False
 
-    def parse(self) -> None:
-        """Parse the source code."""
-        warnings.warn('ModuleAnalyzer.parse() is deprecated.',
-                      RemovedInSphinx50Warning, stacklevel=2)
-        self.analyze()
-
     def analyze(self) -> None:
         """Analyze the source code."""
         if self._analyzed:
-            return None
+            return
 
         try:
             parser = Parser(self.code)
             parser.parse()
 
-            self.attr_docs = OrderedDict()
+            self.attr_docs = {}
             for (scope, comment) in parser.comments.items():
                 if comment:
                     self.attr_docs[scope] = comment.splitlines() + ['']
@@ -172,14 +138,14 @@ class ModuleAnalyzer:
             self.tagorder = parser.deforders
             self._analyzed = True
         except Exception as exc:
-            raise PycodeError('parsing %r failed: %r' % (self.srcname, exc)) from exc
+            raise PycodeError(f'parsing {self.srcname!r} failed: {exc!r}') from exc
 
-    def find_attr_docs(self) -> Dict[Tuple[str, str], List[str]]:
+    def find_attr_docs(self) -> dict[tuple[str, str], list[str]]:
         """Find class and module-level attributes and their documentation."""
         self.analyze()
         return self.attr_docs
 
-    def find_tags(self) -> Dict[str, Tuple[str, int, int]]:
+    def find_tags(self) -> dict[str, tuple[str, int, int]]:
         """Find class, function and method definitions and their location."""
         self.analyze()
         return self.tags
