@@ -7,7 +7,6 @@ import inspect
 import itertools
 import re
 import tokenize
-from collections import OrderedDict
 from inspect import Signature
 from token import DEDENT, INDENT, NAME, NEWLINE, NUMBER, OP, STRING
 from tokenize import COMMENT, NL
@@ -47,7 +46,7 @@ def get_lvar_names(node: ast.AST, self: ast.arg | None = None) -> list[str]:
     node_name = node.__class__.__name__
     if node_name in ('Index', 'Num', 'Slice', 'Str', 'Subscript'):
         raise TypeError('%r does not create new variable' % node)
-    elif node_name == 'Name':
+    if node_name == 'Name':
         if self is None or node.id == self_id:  # type: ignore
             return [node.id]  # type: ignore
         else:
@@ -146,7 +145,7 @@ class TokenProcessor:
 
         return self.current
 
-    def fetch_until(self, condition: Any) -> list[Token]:
+    def fetch_until(self, condition: Any) -> list[Token | None]:
         """Fetch tokens until specified token appeared.
 
         .. note:: This also handles parenthesis well.
@@ -156,7 +155,7 @@ class TokenProcessor:
             tokens.append(self.current)
             if self.current == condition:
                 break
-            elif self.current == [OP, '(']:
+            if self.current == [OP, '(']:
                 tokens += self.fetch_until([OP, ')'])
             elif self.current == [OP, '{']:
                 tokens += self.fetch_until([OP, '}'])
@@ -177,7 +176,7 @@ class AfterCommentParser(TokenProcessor):
         super().__init__(lines)
         self.comment: str | None = None
 
-    def fetch_rvalue(self) -> list[Token]:
+    def fetch_rvalue(self) -> list[Token | None]:
         """Fetch right-hand value of assignment."""
         tokens = []
         while self.fetch_token():
@@ -190,9 +189,9 @@ class AfterCommentParser(TokenProcessor):
                 tokens += self.fetch_until([OP, ']'])
             elif self.current == INDENT:
                 tokens += self.fetch_until(DEDENT)
-            elif self.current == [OP, ';']:
+            elif self.current == [OP, ';']:  # NoQA: SIM114
                 break
-            elif self.current.kind not in (OP, NAME, NUMBER, STRING):
+            elif self.current and self.current.kind not in {OP, NAME, NUMBER, STRING}:
                 break
 
         return tokens
@@ -200,7 +199,7 @@ class AfterCommentParser(TokenProcessor):
     def parse(self) -> None:
         """Parse the code and obtain comment after assignment."""
         # skip lvalue (or whole of AnnAssign)
-        while not self.fetch_token().match([OP, '='], NEWLINE, COMMENT):
+        while (tok := self.fetch_token()) and not tok.match([OP, '='], NEWLINE, COMMENT):
             assert self.current
 
         # skip rvalue (if exists)
@@ -208,7 +207,7 @@ class AfterCommentParser(TokenProcessor):
             self.fetch_rvalue()
 
         if self.current == COMMENT:
-            self.comment = self.current.value
+            self.comment = self.current.value  # type: ignore[union-attr]
 
 
 class VariableCommentPicker(ast.NodeVisitor):
@@ -221,7 +220,7 @@ class VariableCommentPicker(ast.NodeVisitor):
         self.context: list[str] = []
         self.current_classes: list[str] = []
         self.current_function: ast.FunctionDef | None = None
-        self.comments: dict[tuple[str, str], str] = OrderedDict()
+        self.comments: dict[tuple[str, str], str] = {}
         self.annotations: dict[tuple[str, str], str] = {}
         self.previous: ast.AST | None = None
         self.deforders: dict[str, int] = {}
@@ -349,7 +348,7 @@ class VariableCommentPicker(ast.NodeVisitor):
         try:
             targets = get_assign_targets(node)
             varnames: list[str] = sum(
-                [get_lvar_names(t, self=self.get_self()) for t in targets], []
+                [get_lvar_names(t, self=self.get_self()) for t in targets], [],
             )
             current_line = self.get_line(node.lineno)
         except TypeError:
@@ -485,7 +484,7 @@ class DefinitionFinder(TokenProcessor):
             token = self.fetch_token()
             if token is None:
                 break
-            elif token == COMMENT:
+            if token == COMMENT:
                 pass
             elif token == [OP, '@'] and (self.previous is None or
                                          self.previous.match(NEWLINE, NL, INDENT, DEDENT)):
@@ -503,22 +502,23 @@ class DefinitionFinder(TokenProcessor):
     def parse_definition(self, typ: str) -> None:
         """Parse AST of definition."""
         name = self.fetch_token()
-        self.context.append(name.value)
+        self.context.append(name.value)  # type: ignore[union-attr]
         funcname = '.'.join(self.context)
 
         if self.decorator:
             start_pos = self.decorator.start[0]
             self.decorator = None
         else:
-            start_pos = name.start[0]
+            start_pos = name.start[0]  # type: ignore[union-attr]
 
         self.fetch_until([OP, ':'])
-        if self.fetch_token().match(COMMENT, NEWLINE):
+        if self.fetch_token().match(COMMENT, NEWLINE):  # type: ignore[union-attr]
             self.fetch_until(INDENT)
             self.indents.append((typ, funcname, start_pos))
         else:
             # one-liner
-            self.add_definition(funcname, (typ, start_pos, name.end[0]))
+            self.add_definition(funcname,
+                                (typ, start_pos, name.end[0]))  # type: ignore[union-attr]
             self.context.pop()
 
     def finalize_block(self) -> None:
@@ -526,11 +526,11 @@ class DefinitionFinder(TokenProcessor):
         definition = self.indents.pop()
         if definition[0] != 'other':
             typ, funcname, start_pos = definition
-            end_pos = self.current.end[0] - 1
+            end_pos = self.current.end[0] - 1  # type: ignore[union-attr]
             while emptyline_re.match(self.get_line(end_pos)):
                 end_pos -= 1
 
-            self.add_definition(funcname, (typ, start_pos, end_pos))
+            self.add_definition(funcname, (typ, start_pos, end_pos))  # type: ignore[arg-type]
             self.context.pop()
 
 
