@@ -2,8 +2,8 @@
 
 from __future__ import annotations
 
+import sys
 import traceback
-import warnings
 from importlib import import_module
 from types import MethodType
 from typing import TYPE_CHECKING, Any, Callable, Iterator
@@ -15,14 +15,13 @@ from docutils.parsers import Parser
 from docutils.parsers.rst import Directive
 from docutils.transforms import Transform
 
-try:  # Python < 3.10 (backport)
-    from importlib_metadata import entry_points
-except ImportError:
+if sys.version_info >= (3, 10):
     from importlib.metadata import entry_points
+else:
+    from importlib_metadata import entry_points
 
 from sphinx.builders import Builder
 from sphinx.config import Config
-from sphinx.deprecation import RemovedInSphinx70Warning
 from sphinx.domains import Domain, Index, ObjType
 from sphinx.domains.std import GenericObject, Target
 from sphinx.environment import BuildEnvironment
@@ -45,7 +44,7 @@ logger = logging.getLogger(__name__)
 # list of deprecated extensions. Keys are extension name.
 # Values are Sphinx version that merge the extension.
 EXTENSION_BLACKLIST = {
-    "sphinxjp.themecore": "1.2"
+    "sphinxjp.themecore": "1.2",
 }
 
 
@@ -155,18 +154,7 @@ class SphinxComponentRegistry:
         if name not in self.builders:
             raise SphinxError(__('Builder name %s not registered') % name)
 
-        try:
-            return self.builders[name](app, env)
-        except TypeError:
-            warnings.warn(
-                f"The custom builder {name} defines a custom __init__ method without the "
-                f"'env'argument. Report this bug to the developers of your custom builder, "
-                f"this is likely not a issue with Sphinx. The 'env' argument will be required "
-                f"from Sphinx 7.", RemovedInSphinx70Warning, stacklevel=2)
-            builder = self.builders[name](app)
-            if env is not None:
-                builder.set_environment(env)
-            return builder
+        return self.builders[name](app, env)
 
     def add_domain(self, domain: type[Domain], override: bool = False) -> None:
         logger.debug('[app] adding domain: %r', domain)
@@ -203,7 +191,7 @@ class SphinxComponentRegistry:
         directives[name] = cls
 
     def add_role_to_domain(self, domain: str, name: str,
-                           role: RoleFunction | XRefRole, override: bool = False
+                           role: RoleFunction | XRefRole, override: bool = False,
                            ) -> None:
         logger.debug('[app] adding role to domain: %r', (domain, name, role))
         if domain not in self.domains:
@@ -234,7 +222,7 @@ class SphinxComponentRegistry:
         ref_nodeclass: type[TextElement] | None = None,
         objname: str = '',
         doc_field_types: list = [],
-        override: bool = False
+        override: bool = False,
     ) -> None:
         logger.debug('[app] adding object type: %r',
                      (directivename, rolename, indextemplate, parse_node,
@@ -263,7 +251,7 @@ class SphinxComponentRegistry:
         indextemplate: str = '',
         ref_nodeclass: type[TextElement] | None = None,
         objname: str = '',
-        override: bool = False
+        override: bool = False,
     ) -> None:
         logger.debug('[app] adding crossref type: %r',
                      (directivename, rolename, indextemplate, ref_nodeclass, objname))
@@ -286,8 +274,7 @@ class SphinxComponentRegistry:
         logger.debug('[app] adding source_suffix: %r, %r', suffix, filetype)
         if suffix in self.source_suffix and not override:
             raise ExtensionError(__('source_suffix %r is already registered') % suffix)
-        else:
-            self.source_suffix[suffix] = filetype
+        self.source_suffix[suffix] = filetype
 
     def add_source_parser(self, parser: type[Parser], override: bool = False) -> None:
         logger.debug('[app] adding search source_parser: %r', parser)
@@ -297,8 +284,7 @@ class SphinxComponentRegistry:
             if filetype in self.source_parsers and not override:
                 raise ExtensionError(__('source_parser for %r is already registered') %
                                      filetype)
-            else:
-                self.source_parsers[filetype] = parser
+            self.source_parsers[filetype] = parser
 
     def get_source_parser(self, filetype: str) -> type[Parser]:
         try:
@@ -323,8 +309,11 @@ class SphinxComponentRegistry:
             raise ExtensionError(__('Translator for %r already exists') % name)
         self.translators[name] = translator
 
-    def add_translation_handlers(self, node: type[Element],
-                                 **kwargs: tuple[Callable, Callable]) -> None:
+    def add_translation_handlers(
+        self,
+        node: type[Element],
+        **kwargs: tuple[Callable | None, Callable | None],
+    ) -> None:
         logger.debug('[app] adding translation_handlers: %r, %r', node, kwargs)
         for builder_name, handlers in kwargs.items():
             translation_handlers = self.translation_handlers.setdefault(builder_name, {})
@@ -334,16 +323,20 @@ class SphinxComponentRegistry:
             except ValueError as exc:
                 raise ExtensionError(
                     __('kwargs for add_node() must be a (visit, depart) '
-                       'function tuple: %r=%r') % (builder_name, handlers)
+                       'function tuple: %r=%r') % (builder_name, handlers),
                 ) from exc
 
     def get_translator_class(self, builder: Builder) -> type[nodes.NodeVisitor]:
-        return self.translators.get(builder.name,
-                                    builder.default_translator_class)
+        try:
+            return self.translators[builder.name]
+        except KeyError:
+            try:
+                return builder.default_translator_class
+            except AttributeError as err:
+                raise AttributeError(f'translator not found for {builder.name}') from err
 
     def create_translator(self, builder: Builder, *args: Any) -> nodes.NodeVisitor:
         translator_class = self.get_translator_class(builder)
-        assert translator_class, "translator not found for %s" % builder.name
         translator = translator_class(*args)
 
         # transplant handlers for custom nodes to translator instance
@@ -383,7 +376,7 @@ class SphinxComponentRegistry:
     def add_css_files(self, filename: str, **attributes: Any) -> None:
         self.css_files.append((filename, attributes))
 
-    def add_js_file(self, filename: str, **attributes: Any) -> None:
+    def add_js_file(self, filename: str | None, **attributes: Any) -> None:
         logger.debug('[app] adding js_file: %r, %r', filename, attributes)
         self.js_files.append((filename, attributes))
 
@@ -391,7 +384,9 @@ class SphinxComponentRegistry:
         packages = self.latex_packages + self.latex_packages_after_hyperref
         return bool([x for x in packages if x[0] == name])
 
-    def add_latex_package(self, name: str, options: str, after_hyperref: bool = False) -> None:
+    def add_latex_package(
+        self, name: str, options: str | None, after_hyperref: bool = False,
+    ) -> None:
         if self.has_latex_package(name):
             logger.warning("latex package '%s' already included", name)
 
@@ -405,16 +400,19 @@ class SphinxComponentRegistry:
         self,
         node: type[Node],
         figtype: str,
-        title_getter: TitleGetter | None = None, override: bool = False
+        title_getter: TitleGetter | None = None, override: bool = False,
     ) -> None:
         logger.debug('[app] adding enumerable node: (%r, %r, %r)', node, figtype, title_getter)
         if node in self.enumerable_nodes and not override:
             raise ExtensionError(__('enumerable_node %r already registered') % node)
         self.enumerable_nodes[node] = (figtype, title_getter)
 
-    def add_html_math_renderer(self, name: str,
-                               inline_renderers: tuple[Callable, Callable],
-                               block_renderers: tuple[Callable, Callable]) -> None:
+    def add_html_math_renderer(
+        self,
+        name: str,
+        inline_renderers: tuple[Callable | None, Callable | None] | None,
+        block_renderers: tuple[Callable | None, Callable | None] | None,
+    ) -> None:
         logger.debug('[app] adding html_math_renderer: %s, %r, %r',
                      name, inline_renderers, block_renderers)
         if name in self.html_inline_math_renderers:
@@ -459,7 +457,7 @@ class SphinxComponentRegistry:
                     raise VersionRequirementError(
                         __('The %s extension used by this project needs at least '
                            'Sphinx v%s; it therefore cannot be built with this '
-                           'version.') % (extname, err)
+                           'version.') % (extname, err),
                     ) from err
 
             if metadata is None:
@@ -492,7 +490,7 @@ class SphinxComponentRegistry:
 def merge_source_suffix(app: Sphinx, config: Config) -> None:
     """Merge any user-specified source_suffix with any added by extensions."""
     for suffix, filetype in app.registry.source_suffix.items():
-        if suffix not in app.config.source_suffix:
+        if suffix not in app.config.source_suffix:  # NoQA: SIM114
             app.config.source_suffix[suffix] = filetype
         elif app.config.source_suffix[suffix] is None:
             # filetype is not specified (default filetype).
