@@ -6,8 +6,7 @@ import os
 import re
 import sys
 import warnings
-from io import StringIO
-from typing import IO, Any, Generator
+from typing import IO, TYPE_CHECKING, Any, Generator
 from xml.etree import ElementTree
 
 from docutils import nodes
@@ -19,6 +18,9 @@ from sphinx.pycode import ModuleAnalyzer
 from sphinx.testing.path import path
 from sphinx.util.osutil import relpath
 
+if TYPE_CHECKING:
+    from io import StringIO
+
 __all__ = [
     'Struct', 'SphinxTestApp', 'SphinxTestAppWrapperForSkipBuilding',
 ]
@@ -80,7 +82,7 @@ def assert_node(node: Node, cls: Any = None, xpath: str = "", **kwargs: Any) ->
 def etree_parse(path: str) -> Any:
     with warnings.catch_warnings(record=False):
         warnings.filterwarnings("ignore", category=DeprecationWarning)
-        return ElementTree.parse(path)
+        return ElementTree.parse(path)  # NoQA: S314  # using known data in tests
 
 
 class Struct:
@@ -107,11 +109,13 @@ class SphinxTestApp(application.Sphinx):
         warning: IO | None = None,
         tags: list[str] | None = None,
         docutilsconf: str | None = None,
-        parallel: int = 0
+        parallel: int = 0,
     ) -> None:
+        assert srcdir is not None
 
+        self.docutils_conf_path = srcdir / 'docutils.conf'
         if docutilsconf is not None:
-            (srcdir / 'docutils.conf').write_text(docutilsconf)
+            self.docutils_conf_path.write_text(docutilsconf)
 
         if builddir is None:
             builddir = srcdir / '_build'
@@ -152,6 +156,10 @@ class SphinxTestApp(application.Sphinx):
                method not in self._saved_nodeclasses:
                 delattr(nodes.GenericNodeVisitor, 'visit_' + method[6:])
                 delattr(nodes.GenericNodeVisitor, 'depart_' + method[6:])
+        try:
+            os.remove(self.docutils_conf_path)
+        except FileNotFoundError:
+            pass
 
     def __repr__(self) -> str:
         return f'<{self.__class__.__name__} buildername={self.builder.name!r}>'
