@@ -115,7 +115,7 @@ class ReferencesResolver(SphinxPostTransform):
             node.replace_self(newnodes)
 
     def resolve_anyref(
-        self, refdoc: str, node: pending_xref, contnode: Element
+        self, refdoc: str, node: pending_xref, contnode: Element,
     ) -> Element | None:
         """Resolve reference generated by the "any" role."""
         stddomain = self.env.get_domain('std')
@@ -151,7 +151,7 @@ class ReferencesResolver(SphinxPostTransform):
                 reftitle = node.get('reftitle', node.astext())
                 return f':{name}:`{reftitle}`'
             candidates = ' or '.join(stringify(name, role) for name, role in results)
-            logger.warning(__('more than one target found for \'any\' cross-'
+            logger.warning(__("more than one target found for 'any' cross-"
                               'reference %r: could be %s'), target, candidates,
                            location=node)
         res_role, newnode = results[0]
@@ -208,7 +208,7 @@ class ReferencesResolver(SphinxPostTransform):
             msg = __('%r reference target not found: %s') % (typ, target)
         logger.warning(msg, location=node, type='ref', subtype=typ)
 
-    def find_pending_xref_condition(self, node: pending_xref, conditions: Sequence[str]
+    def find_pending_xref_condition(self, node: pending_xref, conditions: Sequence[str],
                                     ) -> list[Node] | None:
         for condition in conditions:
             matched = find_pending_xref_condition(node, condition)
@@ -236,8 +236,13 @@ class SigElementFallbackTransform(SphinxPostTransform):
         def has_visitor(translator: type[nodes.NodeVisitor], node: type[Element]) -> bool:
             return hasattr(translator, "visit_%s" % node.__name__)
 
-        translator = self.app.builder.get_translator_class()
-        if isinstance(translator, SphinxTranslator):
+        try:
+            translator = self.app.builder.get_translator_class()
+        except AttributeError:
+            # do nothing if no translator class is specified (e.g., on a dummy builder)
+            return
+
+        if issubclass(translator, SphinxTranslator):
             # subclass of SphinxTranslator supports desc_sig_element nodes automatically.
             return
 
