@@ -4,11 +4,9 @@ from __future__ import annotations
 
 import os
 import re
-import warnings
 from contextlib import contextmanager
 from copy import copy
 from os import path
-from types import ModuleType
 from typing import IO, TYPE_CHECKING, Any, Callable, Generator, cast
 
 import docutils
@@ -21,7 +19,6 @@ from docutils.statemachine import State, StateMachine, StringList
 from docutils.utils import Reporter, unescape
 from docutils.writers._html_base import HTMLTranslator
 
-from sphinx.deprecation import RemovedInSphinx70Warning, deprecated_alias
 from sphinx.errors import SphinxError
 from sphinx.locale import _, __
 from sphinx.util import logging
@@ -31,20 +28,31 @@ logger = logging.getLogger(__name__)
 report_re = re.compile('^(.+?:(?:\\d+)?): \\((DEBUG|INFO|WARNING|ERROR|SEVERE)/(\\d+)?\\) ')
 
 if TYPE_CHECKING:
+    from types import ModuleType
+
     from docutils.frontend import Values
 
     from sphinx.builders import Builder
     from sphinx.config import Config
     from sphinx.environment import BuildEnvironment
 
-deprecated_alias('sphinx.util.docutils',
-                 {
-                     '__version_info__': docutils.__version_info__,
-                 },
-                 RemovedInSphinx70Warning,
-                 {
-                     '__version_info__': 'docutils.__version_info__',
-                 })
+# deprecated name -> (object to return, canonical path or empty string)
+_DEPRECATED_OBJECTS = {
+    '__version_info__': (docutils.__version_info__, 'docutils.__version_info__'),
+}
+
+
+def __getattr__(name):
+    if name not in _DEPRECATED_OBJECTS:
+        raise AttributeError(f'module {__name__!r} has no attribute {name!r}')
+
+    from sphinx.deprecation import _deprecation_warning
+
+    deprecated_object, canonical_name = _DEPRECATED_OBJECTS[name]
+    _deprecation_warning(__name__, name, canonical_name, remove=(7, 0))
+    return deprecated_object
+
+
 additional_nodes: set[type[Element]] = set()
 
 
@@ -135,7 +143,7 @@ def patched_get_language() -> Generator[None, None, None]:
     """
     from docutils.languages import get_language
 
-    def patched_get_language(language_code: str, reporter: Reporter = None) -> Any:
+    def patched_get_language(language_code: str, reporter: Reporter | None = None) -> Any:
         return get_language(language_code)
 
     try:
@@ -159,7 +167,7 @@ def patched_rst_get_language() -> Generator[None, None, None]:
     """
     from docutils.parsers.rst.languages import get_language
 
-    def patched_get_language(language_code: str, reporter: Reporter = None) -> Any:
+    def patched_get_language(language_code: str, reporter: Reporter | None = None) -> Any:
         return get_language(language_code)
 
     try:
@@ -208,15 +216,15 @@ def du19_footnotes() -> Generator[None, None, None]:
     # Only apply on Docutils 0.18 or 0.18.1, as 0.17 and earlier used a <dl> based
     # approach, and 0.19 and later use the fixed approach by default.
     if docutils.__version_info__[:2] == (0, 18):
-        HTMLTranslator.visit_footnote = visit_footnote  # type: ignore[assignment]
-        HTMLTranslator.depart_footnote = depart_footnote  # type: ignore[assignment]
+        HTMLTranslator.visit_footnote = visit_footnote  # type: ignore[method-assign]
+        HTMLTranslator.depart_footnote = depart_footnote  # type: ignore[method-assign]
 
     try:
         yield
     finally:
         if docutils.__version_info__[:2] == (0, 18):
-            HTMLTranslator.visit_footnote = old_visit_footnote  # type: ignore[assignment]
-            HTMLTranslator.depart_footnote = old_depart_footnote  # type: ignore[assignment]
+            HTMLTranslator.visit_footnote = old_visit_footnote  # type: ignore[method-assign]
+            HTMLTranslator.depart_footnote = old_depart_footnote  # type: ignore[method-assign]
 
 
 @contextmanager
@@ -244,7 +252,7 @@ class CustomReSTDispatcher:
         self.enable()
 
     def __exit__(
-        self, exc_type: type[Exception], exc_value: Exception, traceback: Any
+        self, exc_type: type[Exception], exc_value: Exception, traceback: Any,
     ) -> None:
         self.disable()
 
@@ -260,12 +268,13 @@ class CustomReSTDispatcher:
         roles.role = self.role_func
 
     def directive(self,
-                  directive_name: str, language_module: ModuleType, document: nodes.document
+                  directive_name: str, language_module: ModuleType, document: nodes.document,
                   ) -> tuple[type[Directive] | None, list[system_message]]:
         return self.directive_func(directive_name, language_module, document)
 
-    def role(self, role_name: str, language_module: ModuleType, lineno: int, reporter: Reporter
-             ) -> tuple[RoleFunction, list[system_message]]:
+    def role(
+        self, role_name: str, language_module: ModuleType, lineno: int, reporter: Reporter,
+    ) -> tuple[RoleFunction, list[system_message]]:
         return self.role_func(role_name, language_module, lineno, reporter)
 
 
@@ -312,15 +321,16 @@ class sphinx_domains(CustomReSTDispatcher):
         raise ElementLookupError
 
     def directive(self,
-                  directive_name: str, language_module: ModuleType, document: nodes.document
+                  directive_name: str, language_module: ModuleType, document: nodes.document,
                   ) -> tuple[type[Directive] | None, list[system_message]]:
         try:
             return self.lookup_domain_element('directive', directive_name)
         except ElementLookupError:
             return super().directive(directive_name, language_module, document)
 
-    def role(self, role_name: str, language_module: ModuleType, lineno: int, reporter: Reporter
-             ) -> tuple[RoleFunction, list[system_message]]:
+    def role(
+        self, role_name: str, language_module: ModuleType, lineno: int, reporter: Reporter,
+    ) -> tuple[RoleFunction, list[system_message]]:
         try:
             return self.lookup_domain_element('role', role_name)
         except ElementLookupError:
@@ -360,12 +370,6 @@ class NullReporter(Reporter):
         super().__init__('', 999, 4)
 
 
-def is_html5_writer_available() -> bool:
-    warnings.warn('is_html5_writer_available() is deprecated.',
-                  RemovedInSphinx70Warning)
-    return True
-
-
 @contextmanager
 def switch_source_input(state: State, content: StringList) -> Generator[None, None, None]:
     """Switch current source input of state temporarily."""
@@ -374,7 +378,7 @@ def switch_source_input(state: State, content: StringList) -> Generator[None, No
         get_source_and_line = state.memo.reporter.get_source_and_line  # type: ignore
 
         # replace it by new one
-        state_machine = StateMachine([], None)
+        state_machine = StateMachine([], None)  # type: ignore[arg-type]
         state_machine.input_lines = content
         state.memo.reporter.get_source_and_line = state_machine.get_source_and_line  # type: ignore  # noqa: E501
 
@@ -454,7 +458,7 @@ class SphinxRole:
                         #: (from the "role" directive).
 
     def __call__(self, name: str, rawtext: str, text: str, lineno: int,
-                 inliner: Inliner, options: dict = {}, content: list[str] = []
+                 inliner: Inliner, options: dict = {}, content: list[str] = [],
                  ) -> tuple[list[Node], list[system_message]]:
         self.rawtext = rawtext
         self.text = unescape(text)
@@ -488,12 +492,12 @@ class SphinxRole:
         """Reference to the :class:`.Config` object."""
         return self.env.config
 
-    def get_source_info(self, lineno: int = None) -> tuple[str, int]:
+    def get_source_info(self, lineno: int | None = None) -> tuple[str, int]:
         if lineno is None:
             lineno = self.lineno
         return self.inliner.reporter.get_source_and_line(lineno)  # type: ignore
 
-    def set_source_info(self, node: Node, lineno: int = None) -> None:
+    def set_source_info(self, node: Node, lineno: int | None = None) -> None:
         node.source, node.line = self.get_source_info(lineno)
 
     def get_location(self) -> str:
@@ -517,7 +521,7 @@ class ReferenceRole(SphinxRole):
     explicit_title_re = re.compile(r'^(.+?)\s*(?<!\x00)<(.*?)>$', re.DOTALL)
 
     def __call__(self, name: str, rawtext: str, text: str, lineno: int,
-                 inliner: Inliner, options: dict = {}, content: list[str] = []
+                 inliner: Inliner, options: dict = {}, content: list[str] = [],
                  ) -> tuple[list[Node], list[system_message]]:
         # if the first character is a bang, don't cross-reference at all
         self.disabled = text.startswith('!')
