@@ -13,11 +13,22 @@ import types
 import typing
 from functools import cached_property, partial, partialmethod, singledispatchmethod
 from importlib import import_module
-from inspect import (Parameter, isasyncgenfunction, isclass, ismethod,  # noqa: F401
-                     ismethoddescriptor, ismodule)
+from inspect import (  # noqa: F401
+    Parameter,
+    isasyncgenfunction,
+    isclass,
+    ismethod,
+    ismethoddescriptor,
+    ismodule,
+)
 from io import StringIO
-from types import (ClassMethodDescriptorType, MethodDescriptorType, MethodType, ModuleType,
-                   WrapperDescriptorType)
+from types import (
+    ClassMethodDescriptorType,
+    MethodDescriptorType,
+    MethodType,
+    ModuleType,
+    WrapperDescriptorType,
+)
 from typing import Any, Callable, Dict, Mapping, Sequence, cast
 
 from sphinx.pycode.ast import unparse as ast_unparse
@@ -50,13 +61,11 @@ def unwrap_all(obj: Any, *, stop: Callable | None = None) -> Any:
     while True:
         if stop and stop(obj):
             return obj
-        elif ispartial(obj):
+        if ispartial(obj):
             obj = obj.func
         elif inspect.isroutine(obj) and hasattr(obj, '__wrapped__'):
             obj = obj.__wrapped__
-        elif isclassmethod(obj):
-            obj = obj.__func__
-        elif isstaticmethod(obj):
+        elif isclassmethod(obj) or isstaticmethod(obj):
             obj = obj.__func__
         else:
             return obj
@@ -183,9 +192,9 @@ def isclassmethod(obj: Any, cls: Any = None, name: str | None = None) -> bool:
     """Check if the object is classmethod."""
     if isinstance(obj, classmethod):
         return True
-    elif inspect.ismethod(obj) and obj.__self__ is not None and isclass(obj.__self__):
+    if inspect.ismethod(obj) and obj.__self__ is not None and isclass(obj.__self__):
         return True
-    elif cls and name:
+    if cls and name:
         placeholder = object()
         for basecls in getmro(cls):
             meth = basecls.__dict__.get(name, placeholder)
@@ -242,30 +251,28 @@ def isattributedescriptor(obj: Any) -> bool:
     if inspect.isdatadescriptor(obj):
         # data descriptor is kind of attribute
         return True
-    elif isdescriptor(obj):
+    if isdescriptor(obj):
         # non data descriptor
         unwrapped = unwrap(obj)
         if isfunction(unwrapped) or isbuiltin(unwrapped) or inspect.ismethod(unwrapped):
             # attribute must not be either function, builtin and method
             return False
-        elif is_cython_function_or_method(unwrapped):
+        if is_cython_function_or_method(unwrapped):
             # attribute must not be either function and method (for cython)
             return False
-        elif inspect.isclass(unwrapped):
+        if inspect.isclass(unwrapped):
             # attribute must not be a class
             return False
-        elif isinstance(unwrapped, (ClassMethodDescriptorType,
-                                    MethodDescriptorType,
-                                    WrapperDescriptorType)):
+        if isinstance(unwrapped, (ClassMethodDescriptorType,
+                                  MethodDescriptorType,
+                                  WrapperDescriptorType)):
             # attribute must not be a method descriptor
             return False
-        elif type(unwrapped).__name__ == "instancemethod":
+        if type(unwrapped).__name__ == "instancemethod":
             # attribute must not be an instancemethod (C-API)
             return False
-        else:
-            return True
-    else:
-        return False
+        return True
+    return False
 
 
 def is_singledispatch_function(obj: Any) -> bool:
@@ -319,14 +326,13 @@ def isgenericalias(obj: Any) -> bool:
     """Check if the object is GenericAlias."""
     if isinstance(obj, typing._GenericAlias):  # type: ignore
         return True
-    elif (hasattr(types, 'GenericAlias') and  # only for py39+
-          isinstance(obj, types.GenericAlias)):
+    if (hasattr(types, 'GenericAlias')  # only for py39+
+            and isinstance(obj, types.GenericAlias)):
         return True
-    elif (hasattr(typing, '_SpecialGenericAlias') and  # for py39+
-            isinstance(obj, typing._SpecialGenericAlias)):
+    if (hasattr(typing, '_SpecialGenericAlias')  # for py39+
+            and isinstance(obj, typing._SpecialGenericAlias)):
         return True
-    else:
-        return False
+    return False
 
 
 def safe_getattr(obj: Any, name: str, *defargs: Any) -> Any:
@@ -516,7 +522,7 @@ def _should_unwrap(subject: Callable) -> bool:
     return False
 
 
-def signature(subject: Callable, bound_method: bool = False, type_aliases: dict = {}
+def signature(subject: Callable, bound_method: bool = False, type_aliases: dict = {},
               ) -> inspect.Signature:
     """Return a Signature object for the given *subject*.
 
@@ -573,7 +579,7 @@ def signature(subject: Callable, bound_method: bool = False, type_aliases: dict
 
 
 def evaluate_signature(sig: inspect.Signature, globalns: dict | None = None,
-                       localns: dict | None = None
+                       localns: dict | None = None,
                        ) -> inspect.Signature:
     """Evaluate unresolved type annotations in a signature object."""
     def evaluate_forwardref(ref: ForwardRef, globalns: dict, localns: dict) -> Any:
@@ -722,7 +728,7 @@ def signature_from_ast(node: ast.FunctionDef, code: str = '') -> inspect.Signatu
             default = Parameter.empty
         else:
             default = DefaultValue(
-                ast_unparse(defaults[i + posonlyargs], code)  # type: ignore
+                ast_unparse(defaults[i + posonlyargs], code),  # type: ignore
             )
 
         annotation = ast_unparse(arg.annotation, code) or Parameter.empty
@@ -758,7 +764,7 @@ def getdoc(
     attrgetter: Callable = safe_getattr,
     allow_inherited: bool = False,
     cls: Any = None,
-    name: str | None = None
+    name: str | None = None,
 ) -> str | None:
     """Get the docstring for the object.
 
