@@ -19,7 +19,6 @@ from sphinx.util import logging
 
 if TYPE_CHECKING:
     from sphinx.builders import Builder
-    from sphinx.domain import IndexEntry
     from sphinx.environment import BuildEnvironment
     from sphinx.util.tags import Tags
 
@@ -299,8 +298,8 @@ def get_prev_node(node: Node) -> Node | None:
 
 
 def traverse_translatable_index(
-    doctree: Element
-) -> Iterable[tuple[Element, list[IndexEntry]]]:
+    doctree: Element,
+) -> Iterable[tuple[Element, list[tuple[str, str, str, str, str | None]]]]:
     """Traverse translatable index node from a document tree."""
     matcher = NodeMatcher(addnodes.index, inline=False)
     for node in doctree.findall(matcher):  # type: addnodes.index
@@ -311,7 +310,8 @@ def traverse_translatable_index(
         yield node, entries
 
 
-def nested_parse_with_titles(state: Any, content: StringList, node: Node) -> str:
+def nested_parse_with_titles(state: Any, content: StringList, node: Node,
+                             content_offset: int = 0) -> str:
     """Version of state.nested_parse() that allows titles and does not require
     titles to have the same decoration as the calling document.
 
@@ -324,7 +324,7 @@ def nested_parse_with_titles(state: Any, content: StringList, node: Node) -> str
     state.memo.title_styles = []
     state.memo.section_level = 0
     try:
-        return state.nested_parse(content, 0, node, match_titles=1)
+        return state.nested_parse(content, content_offset, node, match_titles=1)
     finally:
         state.memo.title_styles = surrounding_title_styles
         state.memo.section_level = surrounding_section_level
@@ -353,7 +353,7 @@ indextypes = [
 ]
 
 
-def process_index_entry(entry: str, targetid: str
+def process_index_entry(entry: str, targetid: str,
                         ) -> list[tuple[str, str, str, str, str | None]]:
     from sphinx.domains.python import pairindextypes
 
@@ -364,19 +364,23 @@ def process_index_entry(entry: str, targetid: str
     if entry.startswith('!'):
         main = 'main'
         entry = entry[1:].lstrip()
-    for type in pairindextypes:
-        if entry.startswith(type + ':'):
-            value = entry[len(type) + 1:].strip()
-            value = pairindextypes[type] + '; ' + value
+    for index_type in pairindextypes:
+        if entry.startswith(f'{index_type}:'):
+            value = entry[len(index_type) + 1:].strip()
+            value = f'{pairindextypes[index_type]}; {value}'
+            # xref RemovedInSphinx90Warning
+            logger.warning(__('%r is deprecated for index entries (from entry %r). '
+                              "Use 'pair: %s' instead."),
+                           index_type, entry, value, type='index')
             indexentries.append(('pair', value, targetid, main, None))
             break
     else:
-        for type in indextypes:
-            if entry.startswith(type + ':'):
-                value = entry[len(type) + 1:].strip()
-                if type == 'double':
-                    type = 'pair'
-                indexentries.append((type, value, targetid, main, None))
+        for index_type in indextypes:
+            if entry.startswith(f'{index_type}:'):
+                value = entry[len(index_type) + 1:].strip()
+                if index_type == 'double':
+                    index_type = 'pair'
+                indexentries.append((index_type, value, targetid, main, None))
                 break
         # shorthand notation for single entries
         else:
@@ -393,7 +397,7 @@ def process_index_entry(entry: str, targetid: str
 
 
 def inline_all_toctrees(builder: Builder, docnameset: set[str], docname: str,
-                        tree: nodes.document, colorfunc: Callable, traversed: list[str]
+                        tree: nodes.document, colorfunc: Callable, traversed: list[str],
                         ) -> nodes.document:
     """Inline all toctrees in the *tree*.
 
@@ -525,7 +529,7 @@ def make_id(env: BuildEnvironment, document: nodes.document,
     return node_id
 
 
-def find_pending_xref_condition(node: addnodes.pending_xref, condition: str
+def find_pending_xref_condition(node: addnodes.pending_xref, condition: str,
                                 ) -> Element | None:
     """Pick matched pending_xref_condition node up from the pending_xref."""
     for subnode in node:
@@ -536,7 +540,7 @@ def find_pending_xref_condition(node: addnodes.pending_xref, condition: str
 
 
 def make_refnode(builder: Builder, fromdocname: str, todocname: str, targetid: str | None,
-                 child: Node | list[Node], title: str | None = None
+                 child: Node | list[Node], title: str | None = None,
                  ) -> nodes.reference:
     """Shortcut to create a reference node."""
     node = nodes.reference('', '', internal=True)
@@ -585,7 +589,7 @@ def is_smartquotable(node: Node) -> bool:
     for pnode in traverse_parent(node.parent):
         if isinstance(pnode, NON_SMARTQUOTABLE_PARENT_NODES):
             return False
-        elif pnode.get('support_smartquotes', None) is False:
+        if pnode.get('support_smartquotes', None) is False:
             return False
 
     if getattr(node, 'support_smartquotes', None) is False:
