@@ -10,6 +10,8 @@ from sphinx.builders import Builder
 
 
 class XMLWriter(BaseXMLWriter):
+    output: str
+
     def __init__(self, builder: Builder) -> None:
         super().__init__()
         self.builder = builder
@@ -34,7 +36,7 @@ class PseudoXMLWriter(BaseXMLWriter):
     config_section = 'pseudoxml writer'
     config_section_dependencies = ('writers',)
 
-    output = None
+    output: str
     """Final translated form of `document`."""
 
     def __init__(self, builder: Builder) -> None:
